/*
 * ESP8266 RGBW controller
 * Copyright 2020 (c) Mark van Renswoude
 *
 * https://git.x2software.net/pub/RGBWifi
*/
#include <Arduino.h>
#include <ESP8266WiFi.h>
#include <ESP8266mDNS.h>
#include <ESPAsyncWebServer.h>
#include <ESPAsyncTCP.h>
#include <LittleFS.h>

#include "./config.h"
#include "./debug.h"
#include "./global.h"

#include "./main.wifi.h"
#include "./main.led.h"
#include "./main.debug.h"

#include "./server/static.h"
#include "./server/settings.h"
#include "./server/firmware.h"
#include "./server/api.h"


// Forward declarations
void handleNotFound(AsyncWebServerRequest* request);


AsyncWebServer server(80);



void setup()
{
  _dinit();
  _dln("Initializing LED strip");

  strip = new Strip();
  strip->init(5); // TODO ledCount

  currentTime = millis();

  _dln("Starting LittleFS");
  if (!LittleFS.begin())
    _dln("Setup :: failed to mount file system");

  _dln("Reading settings");
  connectionSettings->read();
  systemSettings->read();

  _dln("Setting pin modes");
  pinMode(systemSettings->pinAPButton(), INPUT_PULLUP);
  pinMode(systemSettings->pinLEDAP(), OUTPUT);
  pinMode(systemSettings->pinLEDSTA(), OUTPUT);


  _dln("Setup :: starting initialization sequence");
  for (uint8_t blink = 0; blink <= 1; blink++)
  {
    digitalWrite(systemSettings->pinLEDAP(), HIGH);
    delay(100);
    digitalWrite(systemSettings->pinLEDAP(), LOW);
    delay(100);
  }

  for (uint8_t blink = 0; blink <= 1; blink++)
  {
    digitalWrite(systemSettings->pinLEDSTA(), HIGH);
    delay(100);
    digitalWrite(systemSettings->pinLEDSTA(), LOW);
    delay(100);
  }

  /*
  uint8_t bottomStep = stepsSettings->count() - 1;

  for (uint8_t i = 0; i < InitialisationBlinkCount; i++)
  {
    stairs->set(bottomStep, InitialisationBrightness, InitialisationFadeTime);
    waitForTransition();

    stairs->set(bottomStep, 0, InitialisationFadeTime);
    waitForTransition();
  }
  */

  _dln("Setup :: initializing WiFi");
  WiFi.persistent(false);
  WiFi.mode(WIFI_OFF);

  initDebug();
  initWiFi();

  _dln("Setup :: registering routes");
  registerStaticRoutes(&server);
  registerAPIRoutes(&server);
  registerSettingsRoutes(&server);
  registerFirmwareRoutes(&server);

  _dln("Setup :: starting HTTP server");
  server.onNotFound(handleNotFound);
  server.begin();
}

void loop()
{
  /*
  _dln("Looping");
    delay(1000);

    // set the colors,
    // if they don't match in order, you need to use NeoGrbFeature feature
    strip->SetPixelColor(0, red);
    strip->SetPixelColor(1, green);
    strip->SetPixelColor(2, blue);
    strip->SetPixelColor(3, white);
    strip->SetPixelColor(4, black);
    // the following line demonstrates rgbw color support
    // if the NeoPixels are rgbw types the following line will compile
    // if the NeoPixels are anything else, the following line will give an error
    //strip->SetPixelColor(3, RgbwColor(colorSaturation));
    strip->Show();


    delay(1000);

    // turn off the pixels
    strip->SetPixelColor(0, black);
    strip->SetPixelColor(1, black);
    strip->SetPixelColor(2, black);
    strip->SetPixelColor(3, black);
    strip->SetPixelColor(4, black);
    strip->Show();

    delay(1000);


  return;
  */



  if (shouldReboot || systemSettingsChanged)
  {
    _dln("Loop :: reboot requested, so long and thanks for all the fish!");
    delay(100);
    ESP.restart();
  }

  currentTime = millis();
  updateDebugStatus();


  if (connectionSettingsChanged)
  {
    _dln("Loop :: connection settings changed");
    initWiFi();
    connectionSettingsChanged = false;
  }


  updateWiFi();
  updateLED();
  //updateNTPClient();
  //checkTriggers();

  //stairs->tick();
}


void handleNotFound(AsyncWebServerRequest *request)
{
  _d("HTTP :: not found: "); _dln(request->url());
  request->send(404);
}


/*
#include <Arduino.h>
#include <ESP8266WiFi.h>
#include <WiFiUDP.h>
#include <ESPAsyncWebServer.h>
#include <ESPAsyncTCP.h>
#include <TimeLib.h>
#include <ArduinoJson.h>
#include <EspSaveCrash.h>

extern "C" {
  #include <user_interface.h>
}

#include "./components/PCA9685.h"
#include "./settings/connection.h"

#include "./main.triggers.h"


ADC_MODE(ADC_VCC);




inline void waitForTransition()
{
  while (stairs->inTransition())
  {
    currentTime = millis();
    stairs->tick();
    delay(1);
  }
}



*/